<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xls;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper;
class Font
{
    /**
     * Color index.
     *
     * @var int
     */
    private $colorIndex;
    /**
     * Font.
     *
     * @var \PhpOffice\PhpSpreadsheet\Style\Font
     */
    private $font;
    /**
     * Constructor.
     */
    public function __construct(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font $font)
    {
        $this->colorIndex = 0x7fff;
        $this->font = $font;
    }
    /**
     * Set the color index.
     *
     * @param int $colorIndex
     */
    public function setColorIndex($colorIndex) : void
    {
        $this->colorIndex = $colorIndex;
    }
    /**
     * Get font record data.
     *
     * @return string
     */
    public function writeFont()
    {
        $font_outline = 0;
        $font_shadow = 0;
        $icv = $this->colorIndex;
        // Index to color palette
        if ($this->font->getSuperscript()) {
            $sss = 1;
        } elseif ($this->font->getSubscript()) {
            $sss = 2;
        } else {
            $sss = 0;
        }
        $bFamily = 0;
        // Font family
        $bCharSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Font::getCharsetFromFontName($this->font->getName());
        // Character set
        $record = 0x31;
        // Record identifier
        $reserved = 0x0;
        // Reserved
        $grbit = 0x0;
        // Font attributes
        if ($this->font->getItalic()) {
            $grbit |= 0x2;
        }
        if ($this->font->getStrikethrough()) {
            $grbit |= 0x8;
        }
        if ($font_outline) {
            $grbit |= 0x10;
        }
        if ($font_shadow) {
            $grbit |= 0x20;
        }
        $data = \pack(
            'vvvvvCCCC',
            // Fontsize (in twips)
            $this->font->getSize() * 20,
            $grbit,
            // Colour
            $icv,
            // Font weight
            self::mapBold($this->font->getBold()),
            // Superscript/Subscript
            $sss,
            self::mapUnderline($this->font->getUnderline()),
            $bFamily,
            $bCharSet,
            $reserved
        );
        $data .= StringHelper::UTF8toBIFF8UnicodeShort($this->font->getName());
        $length = \strlen($data);
        $header = \pack('vv', $record, $length);
        return $header . $data;
    }
    /**
     * Map to BIFF5-BIFF8 codes for bold.
     *
     * @param bool $bold
     *
     * @return int
     */
    private static function mapBold($bold)
    {
        if ($bold) {
            return 0x2bc;
            //  700 = Bold font weight
        }
        return 0x190;
        //  400 = Normal font weight
    }
    /**
     * Map of BIFF2-BIFF8 codes for underline styles.
     *
     * @var int[]
     */
    private static $mapUnderline = [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_NONE => 0x0, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLE => 0x1, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLE => 0x2, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLEACCOUNTING => 0x21, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLEACCOUNTING => 0x22];
    /**
     * Map underline.
     *
     * @param string $underline
     *
     * @return int
     */
    private static function mapUnderline($underline)
    {
        if (isset(self::$mapUnderline[$underline])) {
            return self::$mapUnderline[$underline];
        }
        return 0x0;
    }
}
